package com.example.wtf.controller;

import com.example.wtf.exception.ResourceForbidden;
import com.example.wtf.exception.ResourceNotFound;
import com.example.wtf.model.Source;
import com.example.wtf.repository.SourceRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.linkTo;
import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.methodOn;

@RestController
@RequestMapping("/sources")
public class SourceController {

    @Autowired
    private SourceRepository repository;

    @GetMapping("/get")
    public @ResponseBody CollectionModel<EntityModel<Source>> getSources() {
        List<EntityModel<Source>> sources = new ArrayList<>();

        for (Source source : repository.findAll()) {
            EntityModel<Source> sourceResource = EntityModel.of(
                    source,
                    linkTo(methodOn(SourceController.class).getSource(source.getId())).withSelfRel(),
                    linkTo(methodOn(SourceController.class).updateSource(source.getId(), new Source())).withRel("Update source"),
                    linkTo(methodOn(SourceController.class).deleteSource(source.getId())).withRel("Delete source")
            );
            sources.add(sourceResource);
        }

        return CollectionModel.of(
                sources,
                linkTo(methodOn(SourceController.class).addSource(new Source())).withRel("Add Source")
        );
    }

    @GetMapping("/get/{id}")
    public @ResponseBody EntityModel<Source> getSource(@PathVariable Long id) {
        Source source = repository.findById(id)
                .orElseThrow(() -> new ResourceNotFound("Source not found"));

        return EntityModel.of(
                source,
                linkTo(methodOn(SourceController.class).updateSource(source.getId(), new Source())).withRel("Update source"),
                linkTo(methodOn(SourceController.class).deleteSource(source.getId())).withRel("Delete source"),
                linkTo(methodOn(SourceController.class).getSources()).withRel("Sources")
        );
    }

    @PostMapping("/add")
    public @ResponseBody EntityModel<Source> addSource(@RequestBody Source source) {

        if (repository.existsByTitle(source.getTitle())) {
            throw new ResourceForbidden("Source forbidden");
        }

        repository.save(source);

        return EntityModel.of(
                source,
                linkTo(methodOn(SourceController.class).getSource(source.getId())).withSelfRel(),
                linkTo(methodOn(SourceController.class).updateSource(source.getId(), new Source())).withRel("Update source"),
                linkTo(methodOn(SourceController.class).deleteSource(source.getId())).withRel("Delete source"),
                linkTo(methodOn(SourceController.class).getSources()).withRel("Sources")
        );
    }

    @PutMapping("/update/{id}")
    public @ResponseBody EntityModel<Source> updateSource(@PathVariable Long id,
                                                          @RequestBody Source update) {
        Source source = repository.findById(id)
                .orElseThrow(() -> new ResourceNotFound("Source not found"));

        if (update.getTitle() != null && !update.getTitle().isEmpty()) {
            if (repository.existsByTitle(source.getTitle())) {
                throw new ResourceForbidden("Source forbidden");
            }
            source.setTitle(update.getTitle());
        }
        if (update.getDate() != null) {
            source.setDate(update.getDate());
        }

        repository.save(source);

        return EntityModel.of(
                source,
                linkTo(methodOn(SourceController.class).getSource(id)).withSelfRel(),
                linkTo(methodOn(SourceController.class).deleteSource(id)).withRel("Delete source"),
                linkTo(methodOn(SourceController.class).getSources()).withRel("Sources")
        );
    }

    @DeleteMapping("delete/{id}")
    public @ResponseBody EntityModel<Source> deleteSource(@PathVariable Long id) {
        Source source = repository.findById(id)
                .orElseThrow(() -> new ResourceNotFound("Source not found"));

        repository.delete(source);

        return EntityModel.of(
                source,
                linkTo(methodOn(SourceController.class).getSources()).withRel("Sources")
        );
    }
}